//
//  OpenGL Demonstrator
//  Declaration of the Son Goku/Snowman interactive display demonstrator C++ classes
//
//  Created by brisson on 29/05/04.
//  Copyright (c) 2006-2013 J M. Brisson Lopes & Instituto Superior Tecnico. All rights reserved.
//

#pragma once

#include <string>
using std::string;   // Just use string, no need for the whole std namespace

class CallLists {
public:
    CallLists();
    ~CallLists();
    void       Draw( const int elapsedMilis);
    const bool SwapModel();
protected:
    unsigned int SnowManCallList;
    unsigned int GokuCallList;
    unsigned int FloorList;
    float        ModelAngle;
    bool         drawSnowMan;
    static float ModelSpeed;
};


class Camera {
public:
    Camera();
    const bool SelectCamera( const int cameraType);
    const int  CameraNo() const { return mCameraType; }
    const bool SetProjection() const;
    const bool SetPosition( const int elapsedMilis);
protected:
    int   mCameraType;
    float mCameraAngle;
    static int    minCameraType, maxCameraType;
    static float fov, cameraSpeed;
};


class LightSource {
public:
    LightSource();
    const bool ToggleLightSourceType();
    const bool ToggleLightSourceMovement();
    const bool ToggleGlobalAmbient();
    const bool GlobalAmbientOn() const { return mGlobalAmbientOn; }
    const bool SpotLightOn() const { return mSpotLightSource; }
    const bool PositionLightSource( const int elapsedMilis);
protected:
    bool  mSpotLightSource;
    bool  mMovingLightSource;
    bool  mGlobalAmbientOn;
    float mLightSourceAngle;
    int   SpotList;
    static float lightsourceSpeed;
    static float lightsourcePosition[];
    static float globalAmbient[], noGlobalAmbient[];
};


class OpenGLApplication {
    
public:
    OpenGLApplication();
    ~OpenGLApplication();
    void Initialize( int argc, char ** argv);
    void MyDisplay();
    void MyReshape( const int w, const int h);
    void MyKeyboard( const unsigned char key, const int x, const int y);
    void MyIdle();
    void MyTimer( const int value);
    
    static OpenGLApplication MyApplication;
protected:
    LightSource * MyLightSource;
    Camera      * MyCamera;
    CallLists   * MyLists;
    bool          MyTimerOn;
    bool          FullScreenOn;
    bool          HUDon;
    bool          AxesOn;
    int           lastTimeMilis;
    int           MyFrameCounter, LastFrameRateUpdateTime;
    float         MyFrameRate;
    void          HUD();
    void          DisplayString( const int x, const int y, const string & label, const void * font = NULL);
    static void   display() { MyApplication.MyDisplay(); }
    static void   reshape( int w, int h) { MyApplication.MyReshape( w, h); }
    static void   keyboard( unsigned char key, int x, int y) { MyApplication.MyKeyboard( key, x, y); }
    static void   idle() { MyApplication.MyIdle(); }
    static void   timer( int value ) { MyApplication.MyTimer( value); }
    static int    timerInterval;
    static string commandLabel1, commandLabel2;
};

